"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Rebaser = void 0;
const path_1 = require("path");
const url_1 = require("url");
const source_map_1 = require("source-map");
const convert_source_map_1 = require("convert-source-map");
const events_1 = require("events");
class Rebaser extends events_1.EventEmitter {
    constructor(options = {}) {
        super();
        this._options = options;
    }
    rebase(css, map) {
        const isRebasable = (url) => {
            return !(0, path_1.isAbsolute)(url.href) && (url.host === null) && ((url.hash === null) || (url.path !== null));
        };
        return new Promise((resolve, reject) => {
            try {
                let originalCss = css.toString();
                let rebase = this._options.rebase;
                if (!map) {
                    const converter = (0, convert_source_map_1.fromSource)(originalCss);
                    if (converter) {
                        map = converter.toObject();
                    }
                }
                if (!map) {
                    throw new Error("A map is required, either inline or explicitly passed as an option.");
                }
                let urlsToRebase = [];
                let urlRegExp = /url\('(.*?)'\)|url\("(.*?)"\)|url\((.*?)\)/mg;
                new source_map_1.SourceMapConsumer(map).then((consumer) => {
                    let sourceMapNode = source_map_1.SourceNode.fromStringWithSourceMap(originalCss, consumer);
                    const handleNode = (node) => {
                        if (node.children) {
                            for (let child of node.children) {
                                if (typeof child === 'string') {
                                    let match;
                                    while ((match = urlRegExp.exec(child)) != null) {
                                        let urlStr = match[1] || match[2] || match[3];
                                        let url = (0, url_1.parse)(urlStr);
                                        if (isRebasable(url)) {
                                            let nodeSource = (0, url_1.fileURLToPath)(node.source);
                                            nodeSource = (0, path_1.relative)('.', nodeSource);
                                            let resolvedPath = path_1.posix.join((0, path_1.dirname)(nodeSource), url.pathname);
                                            const done = (rebasedPath) => {
                                                if (rebasedPath !== false) {
                                                    if (!rebasedPath) {
                                                        rebasedPath = resolvedPath;
                                                    }
                                                    urlsToRebase.push({
                                                        sourceMapNode: node,
                                                        originalPath: urlStr,
                                                        rebasedPath: rebasedPath,
                                                        resolvedPath: resolvedPath
                                                    });
                                                }
                                            };
                                            if (!rebase) {
                                                rebase = (node, path, done) => {
                                                    done();
                                                };
                                            }
                                            rebase(nodeSource, resolvedPath, done);
                                        }
                                    }
                                }
                                handleNode(child);
                            }
                        }
                    };
                    handleNode(sourceMapNode);
                    for (let urlToRebase of urlsToRebase) {
                        urlToRebase.sourceMapNode.replaceRight(urlToRebase.originalPath, urlToRebase.rebasedPath);
                        this.emit("rebase", urlToRebase.rebasedPath, urlToRebase.resolvedPath);
                    }
                    let codeWithSourceMap = sourceMapNode.toStringWithSourceMap();
                    resolve({
                        css: Buffer.from(codeWithSourceMap.code),
                        map: Buffer.from(codeWithSourceMap.map.toString())
                    });
                }).catch((error) => {
                    reject(error);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
}
exports.Rebaser = Rebaser;
